import autopath
from py.test import raises

from tiramisu.setting import owners
from tiramisu.config import Config
from tiramisu.option import ChoiceOption, BoolOption, IntOption, FloatOption, \
    StrOption, OptionDescription
from tiramisu.error import ConfigError


def make_description():
    gcoption = ChoiceOption('name', 'GC name', ['ref', 'framework'], 'ref')
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    objspaceoption = ChoiceOption('objspace', 'Object space',
                                  ['std', 'thunk'], 'std')
    booloption = BoolOption('bool', 'Test boolean option', default=True)
    intoption = IntOption('int', 'Test int option', default=0)
    floatoption = FloatOption('float', 'Test float option', default=2.3)
    stroption = StrOption('str', 'Test string option', default="abc")
    boolop = BoolOption('boolop', 'Test boolean option op', default=True)
    wantref_option = BoolOption('wantref', 'Test requires', default=False)
    wantframework_option = BoolOption('wantframework', 'Test requires',
                                      default=False)

    gcgroup = OptionDescription('gc', '', [gcoption, gcdummy, floatoption])
    descr = OptionDescription('tiram', '', [gcgroup, booloption, objspaceoption,
                                            wantref_option, stroption,
                                            wantframework_option,
                                            intoption, boolop])
    return descr


def test_default_owner():
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    descr = OptionDescription('tiramisu', '', [gcdummy])
    cfg = Config(descr)
    assert cfg.dummy is False
    assert cfg.getowner(gcdummy) == 'default'
    cfg.dummy = True
    assert cfg.getowner(gcdummy) == owners.user


def test_addowner():
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    descr = OptionDescription('tiramisu', '', [gcdummy])
    cfg = Config(descr)
    assert cfg.dummy is False
    assert cfg.getowner(gcdummy) == 'default'
    owners.addowner("gen_config")
    cfg.cfgimpl_get_settings().setowner(owners.gen_config)
    cfg.dummy = True
    assert cfg.getowner(gcdummy) == owners.gen_config


def test_owner_is_not_a_string():
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    descr = OptionDescription('tiramisu', '', [gcdummy])
    cfg = Config(descr)
    assert cfg.dummy is False
    assert cfg.getowner(gcdummy) == owners.default
    assert cfg.getowner(gcdummy) == 'default'
    assert isinstance(cfg.getowner(gcdummy), owners.Owner)
    cfg.dummy = True
    assert cfg.getowner(gcdummy) == 'user'


def test_setowner_without_valid_owner():
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    descr = OptionDescription('tiramisu', '', [gcdummy])
    cfg = Config(descr)
    assert cfg.dummy is False
    assert cfg.getowner(gcdummy) == 'default'
    raises(TypeError, "cfg.cfgimpl_get_settings().setowner('gen_config')")


def test_setowner_for_value():
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    descr = OptionDescription('tiramisu', '', [gcdummy])
    cfg = Config(descr)
    assert cfg.dummy is False
    assert cfg.getowner(gcdummy) == 'default'
    owners.addowner("new")
    raises(ConfigError, "cfg.cfgimpl_get_values().setowner(gcdummy, owners.new)")
    cfg.dummy = False
    assert cfg.getowner(gcdummy) == owners.user
    cfg.cfgimpl_get_values().setowner(gcdummy, owners.new)
    assert cfg.getowner(gcdummy) == owners.new
    raises(TypeError, "cfg.cfgimpl_get_values().setowner(gcdummy, 'new')")
